// USB_UART_Bridging.cpp : Defines the entry point for the console application.
//
#include <stdio.h>
#include "stdafx.h"

#include <iostream>
#include <iomanip>
#include <conio.h>
#include <vector>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <string>

// MPLABConnect DLL header file
#include "..\..\APIHeader\MPLABConnect.h"

using namespace std;

#define GREEN					0x0A
#define RED						0x0c
#define BLACK					0x00
#define YELLOW					(FOREGROUND_INTENSITY | FOREGROUND_RED | FOREGROUND_GREEN)

//Common
typedef BOOL (*pfMchpUsbGetVersion) (PCHAR );
typedef void (*pfMchpEnableLogging) (INT);
typedef BOOL  (*pfMchpSetPath) (PCHAR);

//Open/Close
typedef INT(*pfMchpUsbGetAllHubs) (PCHAR);
typedef HANDLE(*pfMchpUsbOpen) (CONST UINT);
typedef HANDLE(*pfMchpUsbOpenID) (UINT16, UINT16);
typedef HANDLE(*pfMchpUsbHCEOpen) (UINT16, UINT16);
typedef BOOL(*pfMchpUsbClose) (HANDLE);

//Get last error
typedef UINT32(*pfMchpUsbGetLastErr) (HANDLE);

typedef BOOL(*pfMchpUsbUartRead)  (HANDLE, UINT32, UINT8*, UINT32, UINT32 *);
typedef BOOL(*pfMchpUsbUartWrite) (HANDLE, UINT32, UINT8 *);
typedef BOOL(*pfMchpUsbSetUARTBaudrate)(HANDLE, UINT32);
typedef BOOL(*pfMchpUsbEnableUARTBridging) (HANDLE, BOOL);

//Common
pfMchpEnableLogging libMchpEnableLogging;
pfMchpUsbGetVersion libMchpUsbGetVersion;
pfMchpSetPath libMchpSetPath;
 

//Open/Close
pfMchpUsbGetAllHubs libMchpUsbGetAllHubs;
pfMchpUsbOpen libMchpUsbOpen;
pfMchpUsbOpenID libMchpUsbOpenID;
pfMchpUsbHCEOpen libMchpUsbHCEOpen;
pfMchpUsbClose libMchpUsbClose;
pfMchpUsbGetLastErr libMchpUsbGetLastErr;

//UART
pfMchpUsbUartRead libMchpUsbUartRead;
pfMchpUsbUartWrite libMchpUsbUartWrite;
pfMchpUsbSetUARTBaudrate libMchpUsbSetUARTBaudrate;
pfMchpUsbEnableUARTBridging libMchpUsbEnableUARTBridging;

CHAR sztext[2048];

void print_to_console(BOOL bsilent, char *chConsole, unsigned char byColor)
{
	HANDLE hstdin = GetStdHandle(STD_INPUT_HANDLE);
	HANDLE hstdout = GetStdHandle(STD_OUTPUT_HANDLE);
	// Remember how things were when we started
	CONSOLE_SCREEN_BUFFER_INFO csbi;
	WORD   index = 0;


	if (byColor != 0xff)
	{
		GetConsoleScreenBufferInfo(hstdout, &csbi);
		SetConsoleTextAttribute(hstdout, byColor);
	}

	if (!bsilent)
		cout << chConsole << endl;

	if (byColor != 0xff)
	{
		SetConsoleTextAttribute(hstdout, csbi.wAttributes);
	}
}

int main( int argc, char* argv[])
{
	try
	{
		UINT16 wVID = 0x0424, wPID = 0x4002, wHCEVID = 0x0424, wHCEPID = 0x7040;
		BOOL bHubIDPresent = FALSE;
		BOOL bHubOpenIDPresent = FALSE;
		BOOL bHCEOpenPresent = FALSE;
		BYTE hubid = 0;
		BOOL bSilent = FALSE;
		int ArgPos = 0;
		UINT hub_count = 0;
		HANDLE hDevice = INVALID_HANDLE_VALUE;
		DWORD dwError = 0;
		char chToPrint[256] = { 0 };
		HMODULE LoadLib;
		CHAR sztext[2048];

		BOOL UARTOperation = FALSE;
		UINT32 BaudRate = 0;
		BOOL SET_BAUDRATE = FALSE;
		UINT32 BytesToWrite = 0;
		UINT8 OutputData[256];
		BOOL UART_WRITE = FALSE;
		BOOL UART_READ = FALSE;
		UINT32 Timeout = 0;
		UINT32 BytesToRead = 0;
		UINT8 InputData[256];
		UINT32 BytesRead =0;

		
		if ((argc <= 1) || (0 == strcmp(argv[1], "--help")) || (0 == strcmp(argv[1], "/?")) )
		{
			cout << left << setw(20) << "--help" << left << setw(20) << "Prints help message\n\n"
			<< left << setw(20) << "Format" << left << setw(10) << "/uart <BaudRate> <operation> <Length> <Timeout\n"
			<< left << setw(20) << "" << left << setw(10) << "in case of -r> ((/id <idValue>) | (/OpenId OpenIdValue) \n"
			<< left << setw(20) << "" << left << setw(10) << "| (/HCEOpen <HCEOpenId)) \n"
			
			<< left << setw(20) << "Arguments" << left << setw(10) << "BaudRate	: Baud rate to be set\n"
			<< left << setw(20) << "" << left << setw(10) << "Operation	: -w - Write, -r - Read\n"
			<< left << setw(20) << "" << left << setw(10) << "Length	: Bytes to read/write\n"
			<< left << setw(20) << "" << left << setw(10) << "Timeout	: Timeout interval, in milliseconds\n\n"

			<<left<< setw(20)<<"Example1"<< left<<setw(10)<<"UART Read : /uart 9600 -r 5000 /id 0\n"
			<<left<< setw(20)<<"Example2"<< left<<setw(10)<<"UART Write : /uart 9600 -w 2 0x6A 0x6B /id 0\n\n"

			<< left << setw(20) << "***********************Device Open Methods*******************************\n"
			<< left << setw(20) << "/id" << left << setw(20) << "idValue: Index of the hub connected to\n"
			<< left << setw(20) << "" << left << setw(10) <<"the system - MchpUsbGetAllHubs\n"
			<< left << setw(20) << "Example" << left << setw(10) << "/id 0 \n \n"
			<< left << setw(20) << "/OpenId" << left << setw(10) << "OpenIdValue: VID of the Hub <space> PID\n"
			<< left << setw(20) << "" << left << setw(10) <<"of the Hub\n"
			<< left << setw(20) << "Example" << left << setw(10) << "/OpenId 0x0424 0x4002\n\n"
			<< left << setw(20) << "/HCEOpen" << left << setw(10) << "HCEOpenId	: VID of the HCE device \n"
			<< left << setw(20) << "" << left << setw(10) <<"<space> PID of the HCE device\n"
			<< left << setw(20) << "Example" << left << setw(10) << "/HCEOpen 0x0424 0x7040\n\n"
			<< left << setw(20) << "Note" << left << setw(20) << "Position of Arguments should be \n"
			<< left << setw(20) << "" << left << setw(10) << "same as Format mentioned in help.\n";

			return 0;
		}
		else
		{
			if (0 == strcmp(argv[1],"/uart"))
			{
				if ((argv[2] != NULL) && (argv[3] != NULL) && (argv[4] != NULL) && (argv[5] != NULL))
				{
					BaudRate = atoi(argv[2]);

					if (0 == strcmp(argv[3], "-w"))
					{
						UARTOperation = TRUE;
						BytesToWrite = atoi(argv[4]);
						for (int i = 0; i < BytesToWrite; i++)
						{
							if (argv[5 + i] != NULL)
							{
								OutputData[i] = (BYTE)strtol(argv[5 + i], 0, 16);
							}
							else
							{
								print_to_console(bSilent, "Data does not match with Length", RED);
								return -1;
							}
						}
						ArgPos = 5 + BytesToWrite;
					}
					else if (0 == strcmp(argv[3], "-r"))
					{
						UARTOperation = FALSE;
						BytesToRead = atoi(argv[4]);
						Timeout = atoi(argv[5]);
						ArgPos = 6;
					}
					else
					{
						print_to_console(bSilent, "Please enter Valid Operation to Perform. -w : WRITE and -r : READ", RED);
						return -1;
					}
				}
				if (argv[ArgPos] != NULL)
				{
					if (0 == strcmp(argv[ArgPos], "/id")) //Device Open Using MchpUsbOpen
					{
						if (argv[ArgPos + 1] != NULL)
						{
							hubid = atoi(argv[ArgPos + 1]);
							bHubIDPresent = TRUE;
						}
						else
						{
							print_to_console(bSilent, "Please enter valid Hub id", RED);
							return -1;
						}
					}
					else if (0 == strcmp(argv[ArgPos], "/OpenId")) //Device Open Using MchpUsbOpenID
					{
						if ((argv[ArgPos + 1] != NULL) && (argv[ArgPos + 2] != NULL))
						{
							wVID = (WORD)strtol(argv[ArgPos + 1], 0, 16);
							wPID = (WORD)strtol(argv[ArgPos + 2], 0, 16);
							bHubOpenIDPresent = TRUE;
						}
						else
						{
							print_to_console(bSilent, "Please enter valid Hub VID and Hub PID", RED);
							return -1;
						}
					}
					else if (0 == strcmp(argv[ArgPos], "/HCEOpen")) //Device Open Using MchpUsbHCEOpen
					{
						if ((argv[ArgPos + 1] != NULL) && (argv[ArgPos + 2] != NULL))
						{
							wHCEVID = (WORD)strtol(argv[ArgPos + 1], 0, 16);
							wHCEPID = (WORD)strtol(argv[ArgPos + 2], 0, 16);
							bHCEOpenPresent = TRUE;
						}
						else
						{
							print_to_console(bSilent, "Please enter valid HFC VID and HFC PID", RED);
							return -1;
						}
					}
				}
				else
				{
					print_to_console(bSilent, "Please enter valid arguments. Refer ReadMe.txt or --help for more details", RED);
					return -1;
				}
			}
		}
		
		TCHAR basepath[MAX_PATH] = { 0 };
		std::string tempfile = "";
		std::string filepath = "";
		GetCurrentDirectory(MAX_PATH, basepath);
		tempfile = basepath;
		filepath = basepath;


		#ifdef x64
			filepath = filepath + "\\APILib\\64-bit\\MPLABConnect.dll";
			LoadLib = LoadLibrary(filepath.c_str());
		#else
			filepath = filepath + "\\APILib\\32-bit\\MPLABConnect.dll";
			LoadLib = LoadLibrary(filepath.c_str());
		#endif

		if (LoadLib != NULL)
		{
			print_to_console(bSilent, "Loadlib library Loaded\n", GREEN);
		}
		else
		{
			sprintf(chToPrint,"Failed to Load DLL : %s\n",filepath.c_str());
			print_to_console(bSilent,chToPrint,RED);
			return -1;
		}

		//Common
		libMchpEnableLogging    = (pfMchpEnableLogging) GetProcAddress ( LoadLib ,"MchpEnableLogging");
		libMchpSetPath    = (pfMchpSetPath) GetProcAddress ( LoadLib ,"MchpSetPath");
		libMchpUsbGetVersion    = (pfMchpUsbGetVersion) GetProcAddress ( LoadLib, "MchpUsbGetVersion");

		//Open/close
		libMchpUsbGetAllHubs		= (pfMchpUsbGetAllHubs) GetProcAddress ( LoadLib ,"MchpUsbGetAllHubs");
		libMchpUsbOpen				= (pfMchpUsbOpen) GetProcAddress ( LoadLib ,"MchpUsbOpen");
		libMchpUsbOpenID				= (pfMchpUsbOpenID) GetProcAddress ( LoadLib ,"MchpUsbOpenID");
		libMchpUsbHCEOpen = (pfMchpUsbHCEOpen) GetProcAddress ( LoadLib ,"MchpUsbHCEOpen");
		libMchpUsbClose			= (pfMchpUsbClose) GetProcAddress ( LoadLib ,"MchpUsbClose");

		libMchpUsbUartRead = (pfMchpUsbUartRead)GetProcAddress(LoadLib, "MchpUsbUartRead");
		libMchpUsbUartWrite = (pfMchpUsbUartWrite)GetProcAddress(LoadLib, "MchpUsbUartWrite");
		libMchpUsbSetUARTBaudrate = (pfMchpUsbSetUARTBaudrate)GetProcAddress(LoadLib, "MchpUsbSetUARTBaudrate");
		libMchpUsbEnableUARTBridging = (pfMchpUsbEnableUARTBridging)GetProcAddress(LoadLib, "MchpUsbEnableUARTBridging");

		libMchpUsbGetLastErr	= (pfMchpUsbGetLastErr) GetProcAddress ( LoadLib ,"MchpUsbGetLastErr");

		//Enable logging
		libMchpEnableLogging(LOG_DETAILED_DESCRIPTION);

		//Get Version
		if (FALSE == libMchpUsbGetVersion(sztext))
		{
			print_to_console(bSilent,"Failed to get Version number", RED);
			return -1;
		}
		else
		{
			sprintf(chToPrint,"Version : %s\n",sztext);
			print_to_console(bSilent,chToPrint,0xFF);
		}
		
		//Set Ini file path
		tempfile = tempfile + "\\APILib\\";
		if(FALSE == libMchpSetPath((PCHAR)tempfile.c_str()))
		{
			sprintf(chToPrint,"File Does not exists : %s\n",tempfile.c_str());
			print_to_console(bSilent,chToPrint,0xFF);
		}

		//Device Open Using MchpUsbOpen
		if (bHubIDPresent)
		{
			hub_count = libMchpUsbGetAllHubs(sztext); //GET all HUBs
			if (hub_count == 0)
			{
				print_to_console(bSilent, "Zero hubs found", RED);
				return -1;
			}
			else
			{
				print_to_console(FALSE,sztext,0xFF);
			}
			hDevice = libMchpUsbOpen(hubid);
			if (hDevice == INVALID_HANDLE_VALUE)
			{
				print_to_console(bSilent, "FAIL: MchpUsbOpen", RED);
				return -1;
			}
			else
			{
				print_to_console(bSilent, "SUCCESS: MchpUsbOpen", GREEN);
			}
		}
		else if (bHubOpenIDPresent) //Device Open Using MchpUsbOpenID
		{
			hDevice = libMchpUsbOpenID(wVID, wPID); //valid handle test
			if (INVALID_HANDLE_VALUE == hDevice)
			{
				print_to_console(bSilent, "FAIL: MchpUsbOpenID", RED);
				return -1;
			}
			else
			{
				print_to_console(bSilent, "SUCCESS: MchpUsbOpenID", GREEN);
			}
		}
		else if (bHCEOpenPresent) //Device Open Using MchpUsbHCEOpen
		{
			hDevice = libMchpUsbHCEOpen(wHCEVID, wHCEPID); //valid handle test
			if (INVALID_HANDLE_VALUE == hDevice)
			{
				print_to_console(bSilent, "FAIL: MchpUsbHCEOpen", RED);
				return -1;
			}
			else
			{
				print_to_console(bSilent, "SUCCESS: MchpUsbHCEOpen", GREEN);
			}
		}
		else
		{
			print_to_console(bSilent,"Please provide valid method to Open the Device",RED);
			return -1;
		}


		if (FALSE == libMchpUsbEnableUARTBridging(hDevice, 1))
		{
			dwError = libMchpUsbGetLastErr(hDevice);

			memset(chToPrint, 0x00, sizeof(chToPrint));
			sprintf(chToPrint, "FAIL: MchpUsbEnableUARTBridging.\n Error: %04xn", dwError);
			print_to_console(bSilent, chToPrint, RED);
			goto Device_Close;
		}
		else
		{
			print_to_console(bSilent, "SUCCESS: MchpUsbEnableUARTBridging - Enabled UART interface", GREEN);
		}
				
		if (FALSE == libMchpUsbSetUARTBaudrate(hDevice, BaudRate))
		{
			dwError = libMchpUsbGetLastErr(hDevice);

			memset(chToPrint, 0x00, sizeof(chToPrint));
			sprintf(chToPrint, "FAIL: MchpUsbSetUARTBaudrate.\n Error: %04xn", dwError);
			print_to_console(bSilent, chToPrint, RED);
			goto Device_Close;
		}
		else
		{
			print_to_console(bSilent, "SUCCESS: MchpUsbSetUARTBaudrate", GREEN);
		}
		

		if (UARTOperation == TRUE)
		{
			if (FALSE == libMchpUsbUartWrite(hDevice, BytesToWrite, OutputData))  //check
			{
				dwError = libMchpUsbGetLastErr(hDevice);
				memset(chToPrint, 0x00, sizeof(chToPrint));
				sprintf(chToPrint, "FAIL: MchpUsbUartWrite.\n Error: %04xn", dwError);
				print_to_console(bSilent, chToPrint, RED);
				goto Device_Close;
			}
			else
			{
				print_to_console(bSilent, "SUCCESS: MchpUsbUartWrite", GREEN);
			}
		}
		else
		{
			if (FALSE == libMchpUsbUartRead(hDevice, BytesToRead, InputData, Timeout, &BytesRead)) 
			{
				dwError = libMchpUsbGetLastErr(hDevice);
				memset(chToPrint, 0x00, sizeof(chToPrint));
				sprintf(chToPrint, "FAIL: MchpUsbUartRead.\n Error: %04xn", dwError);
				print_to_console(bSilent, chToPrint, RED);
				goto Device_Close;
			}
			else
			{
				memset(sztext,0x00,sizeof(sztext));
				print_to_console(FALSE,"Read Data ...",GREEN);
				for ( int i = 0; i <BytesToRead; i++)
				{
					sprintf(chToPrint,"0x%02x ",InputData[i]);
					strcat(sztext,chToPrint);
				}
				print_to_console(FALSE,sztext,YELLOW);

				print_to_console(bSilent, "SUCCESS: MchpUsbUartRead", GREEN);
			}
		}

	Device_Close:
		if (FALSE == libMchpUsbEnableUARTBridging(hDevice, 0))
		{
			dwError = libMchpUsbGetLastErr(hDevice);

			memset(chToPrint, 0x00, sizeof(chToPrint));
			sprintf(chToPrint, "FAIL: MchpUsbEnableUARTBridging.\n Error: %04xn", dwError);
			print_to_console(bSilent, chToPrint, RED);
		}
		else
		{
			print_to_console(bSilent, "SUCCESS: MchpUsbEnableUARTBridging - Disabled UART interface.", GREEN);
		}
		
		if (FALSE == libMchpUsbClose(hDevice))
		{
			dwError = libMchpUsbGetLastErr(hDevice);
			memset(chToPrint, 0x00, sizeof(chToPrint));
			sprintf(chToPrint, "\n FAIL: MchpUsbClose.Error: %04x", dwError);
			print_to_console(bSilent, chToPrint, RED);
		}
		else
		{
			print_to_console(bSilent, "Success: MchpUsbClose", BLACK);
		}
	}
	//Try catch implementation 
	catch (...)
	{
		cout << "Oops... There is a problem with previous command\n" << endl;
		cout << "See log for more details in the application directory\n" << endl;
		return -1;
	}
	return -1;
}

